#include <QCoreApplication>
#include <QTimer>
//#include <iostream>
#include "lib/mcq_error.h"
#include "masschroq_cli.h"
#include <pappsomspp/pappsoexception.h>

// using namespace std;

class MyAppMcq : public QCoreApplication
{
  public:
  MyAppMcq(int &argc, char **argv) : QCoreApplication(argc, argv){};

  private:
  bool
  notify(QObject *receiver_, QEvent *event_)
  {
    try
      {
        return QCoreApplication::notify(receiver_, event_);
      }
    catch(pappso::PappsoException &ex)
      {
        std::cerr << "pappso::PappsoException was caught:\n"
                  << ex.qwhat().toStdString() << std::endl;
        throw ex;
      }
    catch(mcqError &ex)
      {
        std::cerr << "mcqError was caught:\n"
                  << ex.qwhat().toStdString() << std::endl;
        throw ex;
      }
    catch(std::exception &ex)
      {
        std::cerr << "std::exception was caught:\n" << ex.what() << std::endl;
        throw ex;
      }

    return false;
  };
};

int
main(int argc, char **argv)
{

  // QTextStream consoleErr(stderr);
  // QTextStream consoleOut(stdout, QIODevice::WriteOnly);
  ConsoleOut::setCout(new QTextStream(stdout, QIODevice::WriteOnly));
  ConsoleOut::setCerr(new QTextStream(stderr, QIODevice::WriteOnly));

  MyAppMcq app(argc, argv);
  QCoreApplication::setApplicationName("MassChroQ");
  QCoreApplication::setApplicationVersion(MASSCHROQ_VERSION);
  QLocale::setDefault(QLocale::system());

  // create the main class
  masschroqCli myMain;
  // connect up the signals

#if QT_VERSION >= 0x050000
  // Qt5 code
  QObject::connect(&myMain, &masschroqCli::finished, &app, &MyAppMcq::quit);
  QObject::connect(
    &app, &MyAppMcq::aboutToQuit, &myMain, &masschroqCli::aboutToQuitApp);
#else
  // Qt4 code
  QObject::connect(&myMain, SIGNAL(finished()), &app, SLOT(quit()));
  QObject::connect(
    &app, SIGNAL(aboutToQuit()), &myMain, SLOT(aboutToQuitApp()));
#endif

  // This code will start the messaging engine in QT and in
  // 10ms it will start the execution in the MainClass.run routine;
  QTimer::singleShot(10, &myMain, SLOT(run()));
  return app.exec();
}
